var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/app/index.ts
import path2 from "node:path";
import { fileURLToPath as fileURLToPath2 } from "node:url";
import { readFileSync } from "node:fs";
import { map } from "lodash-es";
import yargs from "yargs";
import { hideBin } from "yargs/helpers";
import { openDb as openDb2 } from "gtfs";
import express from "express";
import logger from "morgan";
import untildify2 from "untildify";

// src/lib/formatters.ts
import {
  clone,
  find as find2,
  first as first2,
  groupBy as groupBy2,
  last as last2,
  omit,
  sortBy as sortBy2,
  zipObject
} from "lodash-es";
import moment3 from "moment";

// src/lib/time-utils.ts
import moment from "moment";
function fromGTFSTime(timeString) {
  const duration = moment.duration(timeString);
  return moment({
    hour: duration.hours(),
    minute: duration.minutes(),
    second: duration.seconds()
  });
}
function toGTFSTime(time) {
  return time.format("HH:mm:ss");
}
function fromGTFSDate(gtfsDate) {
  return moment(gtfsDate, "YYYYMMDD");
}
function toGTFSDate(date) {
  return moment(date).format("YYYYMMDD");
}
function calendarToCalendarCode(c) {
  if (c.service_id) {
    return c.service_id;
  }
  return `${c.monday}${c.tuesday}${c.wednesday}${c.thursday}${c.friday}${c.saturday}${c.sunday}`;
}
function calendarCodeToCalendar(code) {
  const days2 = [
    "monday",
    "tuesday",
    "wednesday",
    "thursday",
    "friday",
    "saturday",
    "sunday"
  ];
  const calendar = {};
  for (const [index, day] of days2.entries()) {
    calendar[day] = code[index];
  }
  return calendar;
}
function secondsAfterMidnight(timeString) {
  return moment.duration(timeString).asSeconds();
}
function minutesAfterMidnight(timeString) {
  return moment.duration(timeString).asMinutes();
}
function updateTimeByOffset(timeString, offsetSeconds) {
  const newTime = fromGTFSTime(timeString);
  return toGTFSTime(newTime.add(offsetSeconds, "seconds"));
}

// src/lib/utils.ts
import {
  cloneDeep,
  compact,
  countBy,
  entries,
  every as every2,
  find,
  findLast,
  first,
  flatMap as flatMap2,
  flattenDeep,
  flow,
  isEqual,
  groupBy,
  head,
  last,
  maxBy,
  partialRight,
  reduce,
  size,
  some,
  sortBy,
  uniq,
  uniqBy,
  zip
} from "lodash-es";
import {
  getCalendarDates,
  getTrips,
  getTimetableNotesReferences,
  getTimetableNotes,
  getRoutes,
  getCalendars,
  getTimetableStopOrders,
  getStops,
  getStopAttributes,
  getStoptimes,
  getFrequencies,
  getTimetables,
  getTimetablePages,
  getAgencies,
  openDb
} from "gtfs";
import { stringify } from "csv-stringify";
import moment2 from "moment";
import sqlString from "sqlstring";
import toposort from "toposort";

// src/lib/file-utils.ts
import path from "node:path";
import { fileURLToPath } from "node:url";
import copydir from "copy-dir";
import _ from "lodash-es";
import archiver from "archiver";
import beautify from "js-beautify";
import { renderFile } from "pug";
import puppeteer from "puppeteer";
import sanitize from "sanitize-filename";
import untildify from "untildify";
import insane from "insane";
import { marked } from "marked";

// src/lib/template-functions.ts
var template_functions_exports = {};
__export(template_functions_exports, {
  formatHtmlId: () => formatHtmlId,
  formatTripName: () => formatTripName,
  formatTripNameForCSV: () => formatTripNameForCSV,
  getNotesForStop: () => getNotesForStop,
  getNotesForStoptime: () => getNotesForStoptime,
  getNotesForTimetableLabel: () => getNotesForTimetableLabel,
  getNotesForTrip: () => getNotesForTrip,
  hasNotesOrNotices: () => hasNotesOrNotices,
  timetableHasDifferentDays: () => timetableHasDifferentDays,
  timetablePageHasDifferentDays: () => timetablePageHasDifferentDays,
  timetablePageHasDifferentLabels: () => timetablePageHasDifferentLabels
});
import { every } from "lodash-es";
function formatHtmlId(id) {
  return id.replace(/([^\w[\]{}.:-])\s?/g, "");
}
function timetableHasDifferentDays(timetable) {
  return !every(timetable.orderedTrips, (trip, idx) => {
    if (idx === 0) {
      return true;
    }
    return trip.dayList === timetable.orderedTrips[idx - 1].dayList;
  });
}
function timetablePageHasDifferentDays(timetablePage) {
  return !every(timetablePage.consolidatedTimetables, (timetable, idx) => {
    if (idx === 0) {
      return true;
    }
    return timetable.dayListLong === timetablePage.consolidatedTimetables[idx - 1].dayListLong;
  });
}
function timetablePageHasDifferentLabels(timetablePage) {
  return !every(timetablePage.consolidatedTimetables, (timetable, idx) => {
    if (idx === 0) {
      return true;
    }
    return timetable.timetable_label === timetablePage.consolidatedTimetables[idx - 1].timetable_label;
  });
}
function hasNotesOrNotices(timetable) {
  return timetable.requestPickupSymbolUsed || timetable.noPickupSymbolUsed || timetable.requestDropoffSymbolUsed || timetable.noDropoffSymbolUsed || timetable.noServiceSymbolUsed || timetable.interpolatedStopSymbolUsed || timetable.notes.length > 0;
}
function getNotesForTimetableLabel(notes) {
  return notes.filter((note) => !note.stop_id && !note.trip_id);
}
function getNotesForStop(notes, stop) {
  return notes.filter((note) => {
    if (note.trip_id) {
      return false;
    }
    if (note.stop_sequence && !stop.trips.some((trip) => trip.stop_sequence === note.stop_sequence)) {
      return false;
    }
    return note.stop_id === stop.stop_id;
  });
}
function getNotesForTrip(notes, trip) {
  return notes.filter((note) => {
    if (note.stop_id) {
      return false;
    }
    return note.trip_id === trip.trip_id;
  });
}
function getNotesForStoptime(notes, stoptime) {
  return notes.filter((note) => {
    if (!note.trip_id && note.stop_id === stoptime.stop_id && note.show_on_stoptime === 1) {
      return true;
    }
    if (!note.stop_id && note.trip_id === stoptime.trip_id && note.show_on_stoptime === 1) {
      return true;
    }
    return note.trip_id === stoptime.trip_id && note.stop_id === stoptime.stop_id;
  });
}
function formatTripName(trip, index, timetable) {
  let tripName;
  if (timetable.routes.length > 1) {
    tripName = trip.route_short_name;
  } else if (timetable.orientation === "horizontal") {
    if (trip.trip_short_name) {
      tripName = trip.trip_short_name;
    } else {
      tripName = `Run #${index + 1}`;
    }
  }
  if (timetableHasDifferentDays(timetable)) {
    tripName += ` ${trip.dayList}`;
  }
  return tripName;
}
function formatTripNameForCSV(trip, timetable) {
  let tripName = "";
  if (timetable.routes.length > 1) {
    tripName += `${trip.route_short_name} - `;
  }
  if (trip.trip_short_name) {
    tripName += trip.trip_short_name;
  } else {
    tripName += trip.trip_id;
  }
  if (trip.trip_headsign) {
    tripName += ` - ${trip.trip_headsign}`;
  }
  if (timetableHasDifferentDays(timetable)) {
    tripName += ` - ${trip.dayList}`;
  }
  return tripName;
}

// src/lib/file-utils.ts
function getTemplatePath(templateFileName, config2) {
  let fullTemplateFileName = templateFileName;
  if (config2.noHead !== true) {
    fullTemplateFileName += "_full";
  }
  const templatePath = config2.templatePath === void 0 ? path.join(fileURLToPath(import.meta.url), "../../../views/default") : untildify(config2.templatePath);
  return path.join(templatePath, `${fullTemplateFileName}.pug`);
}
function generateFileName(timetable, config2, extension = "html") {
  let filename = timetable.timetable_id;
  for (const route of timetable.routes) {
    filename += isNullOrEmpty(route.route_short_name) ? `_${route.route_long_name.replace(/\s/g, "-")}` : `_${route.route_short_name.replace(/\s/g, "-")}`;
  }
  if (!isNullOrEmpty(timetable.direction_id)) {
    filename += `_${timetable.direction_id}`;
  }
  filename += `_${formatDays(timetable, config2).replace(/\s/g, "")}.${extension}`;
  return sanitize(filename).toLowerCase();
}
async function renderTemplate(templateFileName, templateVars, config2) {
  const templatePath = getTemplatePath(templateFileName, config2);
  const html = await renderFile(templatePath, {
    _,
    md: (text) => insane(marked.parseInline(text)),
    ...template_functions_exports,
    formatRouteColor,
    formatRouteTextColor,
    ...templateVars
  });
  if (config2.beautify === true) {
    return beautify.html_beautify(html, {
      indent_size: 2
    });
  }
  return html;
}

// src/lib/geojson-utils.ts
import { getShapesAsGeoJSON, getStopsAsGeoJSON } from "gtfs";
import { flatMap } from "lodash-es";
import simplify from "@turf/simplify";
import { featureCollection, round } from "@turf/helpers";
var mergeGeojson = (...geojsons) => featureCollection(flatMap(geojsons, (geojson) => geojson.features));
var truncateGeoJSONDecimals = (geojson, config2) => {
  for (const feature of geojson.features) {
    if (feature.geometry.coordinates) {
      if (feature.geometry.type.toLowerCase() === "point") {
        feature.geometry.coordinates = feature.geometry.coordinates.map(
          (number) => round(number, config2.coordinatePrecision)
        );
      } else if (feature.geometry.type.toLowerCase() === "linestring") {
        feature.geometry.coordinates = feature.geometry.coordinates.map(
          (coordinate) => coordinate.map(
            (number) => round(number, config2.coordinatePrecision)
          )
        );
      } else if (feature.geometry.type.toLowerCase() === "multilinestring") {
        feature.geometry.coordinates = feature.geometry.coordinates.map(
          (linestring) => linestring.map(
            (coordinate) => coordinate.map(
              (number) => round(number, config2.coordinatePrecision)
            )
          )
        );
      }
    }
  }
  return geojson;
};
var simplifyGeoJSON = (geojson, config2) => {
  try {
    const simplifiedGeojson = simplify(geojson, {
      tolerance: 1 / 10 ** config2.coordinatePrecision,
      highQuality: true
    });
    return truncateGeoJSONDecimals(simplifiedGeojson, config2);
  } catch {
    config2.logWarning("Unable to simplify geojson");
    return truncateGeoJSONDecimals(geojson, config2);
  }
};
function getTimetableGeoJSON(timetable, config2) {
  const shapesGeojsons = timetable.route_ids.map(
    (routeId) => getShapesAsGeoJSON({
      route_id: routeId,
      direction_id: timetable.direction_id,
      trip_id: timetable.orderedTrips.map((trip) => trip.trip_id)
    })
  );
  const stopsGeojsons = timetable.route_ids.map(
    (routeId) => getStopsAsGeoJSON({
      route_id: routeId,
      direction_id: timetable.direction_id,
      trip_id: timetable.orderedTrips.map((trip) => trip.trip_id)
    })
  );
  const geojson = mergeGeojson(...shapesGeojsons, ...stopsGeojsons);
  return simplifyGeoJSON(geojson, config2);
}
function getAgencyGeoJSON(config2) {
  const shapesGeojsons = getShapesAsGeoJSON();
  const stopsGeojsons = getStopsAsGeoJSON();
  const geojson = mergeGeojson(shapesGeojsons, stopsGeojsons);
  return simplifyGeoJSON(geojson, config2);
}

// package.json
var version = "2.8.0";

// src/lib/utils.ts
var isTimepoint = (stoptime) => {
  if (isNullOrEmpty(stoptime.timepoint)) {
    return !isNullOrEmpty(stoptime.arrival_time) && !isNullOrEmpty(stoptime.departure_time);
  }
  return stoptime.timepoint === 1;
};
var getLongestTripStoptimes = (trips, config2) => {
  const filteredTripStoptimes = trips.map(
    (trip) => trip.stoptimes.filter((stoptime) => {
      if (config2.showOnlyTimepoint === true) {
        return isTimepoint(stoptime);
      }
      return true;
    })
  );
  return maxBy(filteredTripStoptimes, (stoptimes) => size(stoptimes));
};
var findCommonStopId = (trips, config2) => {
  const longestTripStoptimes = getLongestTripStoptimes(trips, config2);
  if (!longestTripStoptimes) {
    return null;
  }
  const commonStoptime = longestTripStoptimes.find((stoptime, idx) => {
    if (idx === 0 && stoptime.stop_id === last(longestTripStoptimes).stop_id) {
      return false;
    }
    if (isNullOrEmpty(stoptime.arrival_time)) {
      return false;
    }
    return every2(
      trips,
      (trip) => trip.stoptimes.find(
        (tripStoptime) => tripStoptime.stop_id === stoptime.stop_id && tripStoptime.arrival_time !== null
      )
    );
  });
  return commonStoptime ? commonStoptime.stop_id : null;
};
var deduplicateTrips = (trips, commonStopId) => {
  const deduplicatedTrips = [];
  for (const trip of trips) {
    if (deduplicatedTrips.length === 0 || trip.stoptimes.length === 0) {
      deduplicatedTrips.push(trip);
      continue;
    }
    const stoptimes = trip.stoptimes.map((stoptime) => stoptime.departure_time);
    const selectedStoptime = commonStopId ? find(trip.stoptimes, {
      stop_id: commonStopId
    }) : trip.stoptimes[0];
    const similarTrips = deduplicatedTrips.filter((trip2) => {
      const stoptime = find(trip2.stoptimes, {
        stop_id: selectedStoptime.stop_id
      });
      if (!stoptime) {
        return false;
      }
      return stoptime.departure_time === selectedStoptime.departure_time;
    });
    const tripIsUnique = every2(similarTrips, (similarTrip) => {
      const similarTripStoptimes = similarTrip.stoptimes.map(
        (stoptime) => stoptime.departure_time
      );
      return !isEqual(stoptimes, similarTripStoptimes);
    });
    if (tripIsUnique) {
      deduplicatedTrips.push(trip);
    }
  }
  return deduplicatedTrips;
};
var sortTrips = (trips, config2) => {
  let sortedTrips;
  let commonStopId;
  if (config2.sortingAlgorithm === "common") {
    commonStopId = findCommonStopId(trips, config2);
    if (commonStopId) {
      sortedTrips = sortTripsByStoptimeAtStop(trips, commonStopId);
    } else {
      sortedTrips = sortTrips(trips, {
        ...config2,
        sortingAlgorithm: "beginning"
      });
    }
  } else if (config2.sortingAlgorithm === "beginning") {
    for (const trip of trips) {
      if (trip.stoptimes.length === 0) {
        continue;
      }
      trip.firstStoptime = timeToSeconds(first(trip.stoptimes).departure_time);
      trip.lastStoptime = timeToSeconds(last(trip.stoptimes).departure_time);
    }
    sortedTrips = sortBy(
      trips,
      ["firstStoptime", "lastStoptime"],
      ["asc", "asc"]
    );
  } else if (config2.sortingAlgorithm === "end") {
    for (const trip of trips) {
      if (trip.stoptimes.length === 0) {
        continue;
      }
      trip.firstStoptime = timeToSeconds(first(trip.stoptimes).departure_time);
      trip.lastStoptime = timeToSeconds(last(trip.stoptimes).departure_time);
    }
    sortedTrips = sortBy(
      trips,
      ["lastStoptime", "firstStoptime"],
      ["asc", "asc"]
    );
  } else if (config2.sortingAlgorithm === "first") {
    const longestTripStoptimes = getLongestTripStoptimes(trips, config2);
    const firstStopId = first(longestTripStoptimes).stop_id;
    sortedTrips = sortTripsByStoptimeAtStop(trips, firstStopId);
  } else if (config2.sortingAlgorithm === "last") {
    const longestTripStoptimes = getLongestTripStoptimes(trips, config2);
    const lastStopId = last(longestTripStoptimes).stop_id;
    sortedTrips = sortTripsByStoptimeAtStop(trips, lastStopId);
  }
  return deduplicateTrips(sortedTrips, commonStopId);
};
var sortTripsByStoptimeAtStop = (trips, stopId) => sortBy(trips, (trip) => {
  const stoptime = find(trip.stoptimes, { stop_id: stopId });
  return stoptime ? timeToSeconds(stoptime.departure_time) : void 0;
});
var getCalendarDatesForTimetable = (timetable, config2) => {
  const calendarDates = getCalendarDates(
    {
      service_id: timetable.service_ids
    },
    [],
    [["date", "ASC"]]
  );
  const start = fromGTFSDate(timetable.start_date);
  const end = fromGTFSDate(timetable.end_date);
  const excludedDates = [];
  const includedDates = [];
  for (const calendarDate of calendarDates) {
    if (moment2(calendarDate.date, "YYYYMMDD").isBetween(start, end)) {
      if (calendarDate.exception_type === 1) {
        includedDates.push(formatDate(calendarDate, config2.dateFormat));
      } else if (calendarDate.exception_type === 2) {
        excludedDates.push(formatDate(calendarDate, config2.dateFormat));
      }
    }
  }
  const includedAndExcludedDates = excludedDates.filter(
    (date) => includedDates.includes(date)
  );
  return {
    excludedDates: excludedDates.filter(
      (date) => !includedAndExcludedDates.includes(date)
    ),
    includedDates: includedDates.filter(
      (date) => !includedAndExcludedDates.includes(date)
    )
  };
};
var getDaysFromCalendars = (calendars) => {
  const days2 = {
    monday: 0,
    tuesday: 0,
    wednesday: 0,
    thursday: 0,
    friday: 0,
    saturday: 0,
    sunday: 0
  };
  for (const calendar of calendars) {
    for (const [day, value] of Object.entries(days2)) {
      days2[day] = value | calendar[day];
    }
  }
  return days2;
};
var getDirectionHeadsignFromTimetable = (timetable) => {
  const trips = getTrips(
    {
      direction_id: timetable.direction_id,
      route_id: timetable.route_ids
    },
    ["trip_headsign"]
  );
  if (trips.length === 0) {
    return "";
  }
  const mostCommonHeadsign = flow(
    countBy,
    entries,
    partialRight(maxBy, last),
    head
  )(compact(trips.map((trip) => trip.trip_headsign)));
  return mostCommonHeadsign;
};
var getTimetableNotesForTimetable = (timetable, config2) => {
  const noteReferences = [
    // Get all notes for this timetable.
    ...getTimetableNotesReferences({
      timetable_id: timetable.timetable_id
    }),
    // Get all notes for this route.
    ...getTimetableNotesReferences({
      route_id: timetable.routes.map((route) => route.route_id),
      timetable_id: null
    }),
    // Get all notes for all trips in this timetable.
    ...getTimetableNotesReferences({
      trip_id: timetable.orderedTrips.map((trip) => trip.trip_id)
    }),
    // Get all notes for all stops in this timetable.
    ...getTimetableNotesReferences({
      stop_id: timetable.stops.map((stop) => stop.stop_id),
      trip_id: null,
      route_id: null,
      timetable_id: null
    })
  ];
  const usedNoteReferences = [];
  for (const noteReference of noteReferences) {
    if (noteReference.stop_sequence === "" || noteReference.stop_sequence === null) {
      usedNoteReferences.push(noteReference);
      continue;
    }
    if (noteReference.stop_id === "" || noteReference.stop_id === null) {
      config2.logWarning(
        `Timetable Note Reference for note_id=${noteReference.note_id} has a \`stop_sequence\` but no \`stop_id\` - ignoring`
      );
      continue;
    }
    const stop = timetable.stops.find(
      (stop2) => stop2.stop_id === noteReference.stop_id
    );
    if (!stop) {
      continue;
    }
    const tripWithMatchingStopSequence = stop.trips.find(
      (trip) => trip.stop_sequence === noteReference.stop_sequence
    );
    if (tripWithMatchingStopSequence) {
      usedNoteReferences.push(noteReference);
    }
  }
  const notes = getTimetableNotes({
    note_id: usedNoteReferences.map((noteReference) => noteReference.note_id)
  });
  const symbols = "abcdefghijklmnopqrstuvwxyz".split("");
  let symbolIndex = 0;
  for (const note of notes) {
    if (note.symbol === "" || note.symbol === null) {
      note.symbol = symbolIndex < symbols.length - 1 ? symbols[symbolIndex] : symbolIndex - symbols.length;
      symbolIndex += 1;
    }
  }
  const formattedNotes = usedNoteReferences.map((noteReference) => ({
    ...noteReference,
    ...notes.find((note) => note.note_id === noteReference.note_id)
  }));
  return sortBy(formattedNotes, "symbol");
};
var convertTimetableToTimetablePage = (timetable, config2) => {
  if (!timetable.routes) {
    timetable.routes = getRoutes({
      route_id: timetable.route_ids
    });
  }
  const filename = generateFileName(timetable, config2, "html");
  return {
    timetable_page_id: timetable.timetable_id,
    timetable_page_label: timetable.timetable_label,
    timetables: [timetable],
    filename
  };
};
var convertRouteToTimetablePage = (route, direction, calendars, calendarDates, config2) => {
  const timetable = {
    route_ids: [route.route_id],
    direction_id: direction ? direction.direction_id : void 0,
    direction_name: direction ? direction.trip_headsign : void 0,
    routes: [route],
    include_exceptions: calendarDates && calendarDates.length > 0 ? 1 : 0,
    service_id: calendarDates && calendarDates.length > 0 ? calendarDates[0].service_id : null,
    service_notes: null,
    timetable_label: null,
    start_time: null,
    end_time: null,
    orientation: null,
    timetable_sequence: null,
    show_trip_continuation: null,
    start_date: null,
    end_date: null
  };
  if (calendars && calendars.length > 0) {
    Object.assign(timetable, getDaysFromCalendars(calendars));
    timetable.start_date = toGTFSDate(
      moment2.min(
        calendars.map((calendar) => fromGTFSDate(calendar.start_date))
      )
    );
    timetable.end_date = toGTFSDate(
      moment2.max(calendars.map((calendar) => fromGTFSDate(calendar.end_date)))
    );
  }
  timetable.timetable_id = formatTimetableId(timetable);
  return convertTimetableToTimetablePage(timetable, config2);
};
var convertRoutesToTimetablePages = (config2) => {
  const db = openDb(config2);
  const routes = getRoutes();
  let whereClause = "";
  const whereClauses = [];
  if (config2.endDate) {
    whereClauses.push(
      `start_date <= ${sqlString.escape(toGTFSDate(moment2(config2.endDate)))}`
    );
  }
  if (config2.startDate) {
    whereClauses.push(
      `end_date >= ${sqlString.escape(toGTFSDate(moment2(config2.startDate)))}`
    );
  }
  if (whereClauses.length > 0) {
    whereClause = `WHERE ${whereClauses.join(" AND ")}`;
  }
  const calendars = db.prepare(`SELECT * FROM calendar ${whereClause}`).all();
  const serviceIds = calendars.map((calendar) => calendar.service_id);
  const calendarDates = db.prepare(
    `SELECT * FROM calendar_dates WHERE exception_type = 1 AND service_id NOT IN (${serviceIds.map((serviceId) => `'${serviceId}'`).join(", ")})`
  ).all();
  const timetablePages = routes.map((route) => {
    const trips = getTrips(
      {
        route_id: route.route_id
      },
      ["trip_headsign", "direction_id", "trip_id", "service_id"]
    );
    const directions = uniqBy(trips, (trip) => trip.direction_id);
    const dayGroups = groupBy(calendars, calendarToCalendarCode);
    const calendarDateGroups = groupBy(calendarDates, "service_id");
    return directions.map((direction) => [
      Object.values(dayGroups).map((calendars2) => {
        const tripsForCalendars = trips.filter(
          (trip) => some(calendars2, { service_id: trip.service_id })
        );
        if (tripsForCalendars.length > 0) {
          return convertRouteToTimetablePage(
            route,
            direction,
            calendars2,
            null,
            config2
          );
        }
      }),
      Object.values(calendarDateGroups).map((calendarDates2) => {
        const tripsForCalendarDates = trips.filter(
          (trip) => some(calendarDates2, { service_id: trip.service_id })
        );
        if (tripsForCalendarDates.length > 0) {
          return convertRouteToTimetablePage(
            route,
            direction,
            null,
            calendarDates2,
            config2
          );
        }
      })
    ]);
  });
  return compact(flattenDeep(timetablePages));
};
var generateTripsByFrequencies = (trip, frequencies, config2) => {
  const formattedFrequencies = frequencies.map(
    (frequency) => formatFrequency(frequency, config2)
  );
  const resetTrip = resetStoptimesToMidnight(trip);
  const trips = [];
  for (const frequency of formattedFrequencies) {
    const startSeconds = secondsAfterMidnight(frequency.start_time);
    const endSeconds = secondsAfterMidnight(frequency.end_time);
    for (let offset = startSeconds; offset < endSeconds; offset += frequency.headway_secs) {
      const newTrip = cloneDeep(resetTrip);
      trips.push({
        ...newTrip,
        trip_id: `${resetTrip.trip_id}_freq_${trips.length}`,
        stoptimes: updateStoptimesByOffset(newTrip, offset)
      });
    }
  }
  return trips;
};
var duplicateStopsForDifferentArrivalDeparture = (stopIds, timetable, config2) => {
  if (config2.showArrivalOnDifference === null) {
    return stopIds;
  }
  for (const trip of timetable.orderedTrips) {
    for (const stoptime of trip.stoptimes) {
      const timepointDifference = fromGTFSTime(stoptime.departure_time).diff(
        fromGTFSTime(stoptime.arrival_time),
        "minutes"
      );
      if (timepointDifference < config2.showArrivalOnDifference) {
        continue;
      }
      const index = stopIds.indexOf(stoptime.stop_id);
      if (index === 0 || index === stopIds.length - 1) {
        continue;
      }
      if (stoptime.stop_id === stopIds[index + 1] || stoptime.stop_id === stopIds[index - 1]) {
        continue;
      }
      stopIds.splice(index, 0, stoptime.stop_id);
    }
  }
  return stopIds;
};
var getStopOrder = (timetable, config2) => {
  const timetableStopOrders = getTimetableStopOrders(
    {
      timetable_id: timetable.timetable_id
    },
    ["stop_id"],
    [["stop_sequence", "ASC"]]
  );
  if (timetableStopOrders.length > 0) {
    return timetableStopOrders.map(
      (timetableStopOrder) => timetableStopOrder.stop_id
    );
  }
  try {
    const stopGraph = [];
    for (const trip of timetable.orderedTrips) {
      const sortedStopIds = trip.stoptimes.filter((stoptime) => {
        if (config2.showOnlyTimepoint === true) {
          return isTimepoint(stoptime);
        }
        return true;
      }).map((stoptime) => stoptime.stop_id);
      for (const [index, stopId] of sortedStopIds.entries()) {
        if (index === sortedStopIds.length - 1) {
          continue;
        }
        stopGraph.push([stopId, sortedStopIds[index + 1]]);
      }
    }
    const stopIds2 = toposort(stopGraph);
    return duplicateStopsForDifferentArrivalDeparture(
      stopIds2,
      timetable,
      config2
    );
  } catch {
  }
  const longestTripStoptimes = getLongestTripStoptimes(
    timetable.orderedTrips,
    config2
  );
  const stopIds = longestTripStoptimes.map((stoptime) => stoptime.stop_id);
  return duplicateStopsForDifferentArrivalDeparture(stopIds, timetable, config2);
};
var getStopsForTimetable = (timetable, config2) => {
  if (timetable.orderedTrips.length === 0) {
    return [];
  }
  const orderedStopIds = getStopOrder(timetable, config2);
  const orderedStops = orderedStopIds.map((stopId, index) => {
    const stops = getStops({
      stop_id: stopId
    });
    if (stops.length === 0) {
      throw new Error(
        `No stop found found for stop_id=${stopId} in timetable_id=${timetable.timetable_id}`
      );
    }
    const stop = {
      ...stops[0],
      trips: []
    };
    if (index < orderedStopIds.length - 1 && stopId === orderedStopIds[index + 1]) {
      stop.type = "arrival";
    } else if (index > 0 && stopId === orderedStopIds[index - 1]) {
      stop.type = "departure";
    }
    return stop;
  });
  if (timetable.showStopCity) {
    const stopAttributes = getStopAttributes({
      stop_id: orderedStopIds
    });
    for (const stopAttribute of stopAttributes) {
      const stop = orderedStops.find(
        (stop2) => stop2.stop_id === stopAttribute.stop_id
      );
      if (stop) {
        stop.stop_city = stopAttribute.stop_city;
      }
    }
  }
  return orderedStops;
};
var getCalendarsFromTimetable = (timetable) => {
  const db = openDb();
  let whereClause = "";
  const whereClauses = [];
  if (timetable.end_date) {
    if (!moment2(timetable.end_date, "YYYYMMDD", true).isValid()) {
      throw new Error(
        `Invalid end_date=${timetable.end_date} for timetable_id=${timetable.timetable_id}`
      );
    }
    whereClauses.push(`start_date <= ${sqlString.escape(timetable.end_date)}`);
  }
  if (timetable.start_date) {
    if (!moment2(timetable.start_date, "YYYYMMDD", true).isValid()) {
      throw new Error(
        `Invalid start_date=${timetable.start_date} for timetable_id=${timetable.timetable_id}`
      );
    }
    whereClauses.push(`end_date >= ${sqlString.escape(timetable.start_date)}`);
  }
  const days2 = getDaysFromCalendars([timetable]);
  const dayQueries = reduce(
    days2,
    (memo, value, key) => {
      if (value === 1) {
        memo.push(`${key} = 1`);
      }
      return memo;
    },
    []
  );
  if (dayQueries.length > 0) {
    whereClauses.push(`(${dayQueries.join(" OR ")})`);
  }
  if (whereClauses.length > 0) {
    whereClause = `WHERE ${whereClauses.join(" AND ")}`;
  }
  return db.prepare(`SELECT * FROM calendar ${whereClause}`).all();
};
var getCalendarDatesServiceIds = (startDate, endDate) => {
  const db = openDb();
  const whereClauses = ["exception_type = 1"];
  if (endDate) {
    whereClauses.push(`date <= ${sqlString.escape(endDate)}`);
  }
  if (startDate) {
    whereClauses.push(`date >= ${sqlString.escape(startDate)}`);
  }
  const calendarDates = db.prepare(
    `SELECT DISTINCT service_id FROM calendar_dates WHERE ${whereClauses.join(
      " AND "
    )}`
  ).all();
  return calendarDates.map((calendarDate) => calendarDate.service_id);
};
var getAllStationStopIds = (stopId) => {
  const stops = getStops({
    stop_id: stopId
  });
  if (stops.length === 0) {
    throw new Error(`No stop found for stop_id=${stopId}`);
  }
  const stop = stops[0];
  if (isNullOrEmpty(stop.parent_station)) {
    return [stopId];
  }
  const stopsInParentStation = getStops(
    {
      parent_station: stop.parent_station
    },
    ["stop_id"]
  );
  return [
    stop.parent_station,
    ...stopsInParentStation.map((stop2) => stop2.stop_id)
  ];
};
var getTripsWithSameBlock = (trip, timetable) => {
  const trips = getTrips(
    {
      block_id: trip.block_id,
      service_id: timetable.service_ids
    },
    ["trip_id", "route_id"]
  );
  for (const blockTrip of trips) {
    const stopTimes = getStoptimes(
      {
        trip_id: blockTrip.trip_id
      },
      [],
      [["stop_sequence", "ASC"]]
    );
    if (stopTimes.length === 0) {
      throw new Error(
        `No stoptimes found found for trip_id=${blockTrip.trip_id}`
      );
    }
    blockTrip.firstStoptime = first(stopTimes);
    blockTrip.lastStoptime = last(stopTimes);
  }
  return sortBy(trips, (trip2) => trip2.firstStoptime.departure_timestamp);
};
var addTripContinuation = (trip, timetable) => {
  if (!trip.block_id || trip.stoptimes.length === 0) {
    return;
  }
  const maxContinuesAsWaitingTimeSeconds = 60 * 60;
  const firstStoptime = first(trip.stoptimes);
  const firstStopIds = getAllStationStopIds(firstStoptime.stop_id);
  const lastStoptime = last(trip.stoptimes);
  const lastStopIds = getAllStationStopIds(lastStoptime.stop_id);
  const blockTrips = getTripsWithSameBlock(trip, timetable);
  const previousTrip = findLast(
    blockTrips,
    (blockTrip) => blockTrip.lastStoptime.arrival_timestamp <= firstStoptime.departure_timestamp
  );
  if (previousTrip && previousTrip.route_id !== trip.route_id && previousTrip.lastStoptime.arrival_timestamp >= firstStoptime.departure_timestamp - maxContinuesAsWaitingTimeSeconds && firstStopIds.includes(previousTrip.lastStoptime.stop_id)) {
    const routes = getRoutes({
      route_id: previousTrip.route_id
    });
    previousTrip.route = routes[0];
    trip.continues_from_route = previousTrip;
  }
  const nextTrip = find(
    blockTrips,
    (blockTrip) => blockTrip.firstStoptime.departure_timestamp >= lastStoptime.arrival_timestamp
  );
  if (nextTrip && nextTrip.route_id !== trip.route_id && nextTrip.firstStoptime.departure_timestamp <= lastStoptime.arrival_timestamp + maxContinuesAsWaitingTimeSeconds && lastStopIds.includes(nextTrip.firstStoptime.stop_id)) {
    const routes = getRoutes({
      route_id: nextTrip.route_id
    });
    nextTrip.route = routes[0];
    trip.continues_as_route = nextTrip;
  }
};
var filterTrips = (timetable) => {
  let filteredTrips = timetable.orderedTrips;
  for (const trip of filteredTrips) {
    const combinedStoptimes = [];
    for (const [index, stoptime] of trip.stoptimes.entries()) {
      if (index === 0 || stoptime.stop_id !== trip.stoptimes[index - 1].stop_id) {
        combinedStoptimes.push(stoptime);
      } else {
        combinedStoptimes[combinedStoptimes.length - 1].departure_time = stoptime.departure_time;
      }
    }
    trip.stoptimes = combinedStoptimes;
  }
  const timetableStopIds = new Set(timetable.stops.map((stop) => stop.stop_id));
  for (const trip of filteredTrips) {
    trip.stoptimes = trip.stoptimes.filter(
      (stoptime) => timetableStopIds.has(stoptime.stop_id)
    );
  }
  filteredTrips = filteredTrips.filter((trip) => trip.stoptimes.length > 1);
  return filteredTrips;
};
var getTripsForTimetable = (timetable, calendars, config2) => {
  const tripQuery = {
    route_id: timetable.route_ids,
    service_id: timetable.service_ids
  };
  if (!isNullOrEmpty(timetable.direction_id)) {
    tripQuery.direction_id = timetable.direction_id;
  }
  const trips = getTrips(tripQuery);
  if (trips.length === 0) {
    timetable.warnings.push(
      `No trips found for route_id=${timetable.route_ids.join(
        "_"
      )}, direction_id=${timetable.direction_id}, service_ids=${JSON.stringify(
        timetable.service_ids
      )}, timetable_id=${timetable.timetable_id}`
    );
  }
  const frequencies = getFrequencies({
    trip_id: trips.map((trip) => trip.trip_id)
  });
  timetable.service_ids = uniq(trips.map((trip) => trip.service_id));
  const formattedTrips = [];
  for (const trip of trips) {
    const formattedTrip = formatTrip(trip, timetable, calendars, config2);
    formattedTrip.stoptimes = getStoptimes(
      {
        trip_id: formattedTrip.trip_id
      },
      [],
      [["stop_sequence", "ASC"]]
    );
    if (formattedTrip.stoptimes.length === 0) {
      timetable.warnings.push(
        `No stoptimes found for trip_id=${formattedTrip.trip_id}, route_id=${timetable.route_ids.join("_")}, timetable_id=${timetable.timetable_id}`
      );
    }
    if (timetable.start_timestamp !== "" && timetable.start_timestamp !== null && timetable.start_timestamp !== void 0 && trip.stoptimes[0].arrival_timestamp < timetable.start_timestamp) {
      return;
    }
    if (timetable.end_timestamp !== "" && timetable.end_timestamp !== null && timetable.end_timestamp !== void 0 && trip.stoptimes[0].arrival_timestamp >= timetable.end_timestamp) {
      return;
    }
    if (timetable.show_trip_continuation) {
      addTripContinuation(formattedTrip, timetable);
      if (formattedTrip.continues_as_route) {
        timetable.has_continues_as_route = true;
      }
      if (formattedTrip.continues_from_route) {
        timetable.has_continues_from_route = true;
      }
    }
    const tripFrequencies = frequencies.filter(
      (frequency) => frequency.trip_id === trip.trip_id
    );
    if (tripFrequencies.length === 0) {
      formattedTrips.push(formattedTrip);
    } else {
      const frequencyTrips = generateTripsByFrequencies(
        formattedTrip,
        frequencies,
        config2
      );
      formattedTrips.push(...frequencyTrips);
      timetable.frequencies = frequencies;
      timetable.frequencyExactTimes = some(frequencies, {
        exact_times: 1
      });
    }
  }
  if (config2.useParentStation) {
    const stopIds = [];
    for (const trip of formattedTrips) {
      for (const stoptime of trip.stoptimes) {
        stopIds.push(stoptime.stop_id);
      }
    }
    const stops = getStops(
      {
        stop_id: uniq(stopIds)
      },
      ["parent_station", "stop_id"]
    );
    for (const trip of formattedTrips) {
      for (const stoptime of trip.stoptimes) {
        const stop = stops.find((stop2) => stop2.stop_id === stoptime.stop_id);
        if (stop?.parent_station) {
          stoptime.stop_id = stop.parent_station;
        }
      }
    }
  }
  return sortTrips(formattedTrips, config2);
};
var formatTimetables = (timetables, config2) => {
  const formattedTimetables = timetables.map((timetable) => {
    timetable.warnings = [];
    const dayList = formatDays(timetable, config2);
    const calendars = getCalendarsFromTimetable(timetable);
    let serviceIds = calendars.map((calendar) => calendar.service_id);
    if (timetable.include_exceptions === 1) {
      const calendarDatesServiceIds = getCalendarDatesServiceIds(
        timetable.start_date,
        timetable.end_date
      );
      serviceIds = uniq([...serviceIds, ...calendarDatesServiceIds]);
    }
    Object.assign(timetable, {
      noServiceSymbolUsed: false,
      requestDropoffSymbolUsed: false,
      noDropoffSymbolUsed: false,
      requestPickupSymbolUsed: false,
      noPickupSymbolUsed: false,
      interpolatedStopSymbolUsed: false,
      showStopCity: config2.showStopCity,
      showStopDescription: config2.showStopDescription,
      noServiceSymbol: config2.noServiceSymbol,
      requestDropoffSymbol: config2.requestDropoffSymbol,
      noDropoffSymbol: config2.noDropoffSymbol,
      requestPickupSymbol: config2.requestPickupSymbol,
      noPickupSymbol: config2.noPickupSymbol,
      interpolatedStopSymbol: config2.interpolatedStopSymbol,
      orientation: timetable.orientation || config2.defaultOrientation,
      service_ids: serviceIds,
      dayList,
      dayListLong: formatDaysLong(dayList, config2)
    });
    timetable.orderedTrips = getTripsForTimetable(timetable, calendars, config2);
    timetable.stops = getStopsForTimetable(timetable, config2);
    timetable.calendarDates = getCalendarDatesForTimetable(timetable, config2);
    timetable.timetable_label = formatTimetableLabel(timetable);
    timetable.notes = getTimetableNotesForTimetable(timetable, config2);
    if (config2.showMap) {
      timetable.geojson = getTimetableGeoJSON(timetable, config2);
    }
    timetable.orderedTrips = filterTrips(timetable);
    timetable.stops = formatStops(timetable, config2);
    return timetable;
  });
  if (config2.allowEmptyTimetables) {
    return formattedTimetables;
  }
  return formattedTimetables.filter(
    (timetable) => timetable.orderedTrips.length > 0
  );
};
function getTimetablePagesForAgency(config2) {
  const timetables = mergeTimetablesWithSameId(getTimetables());
  if (timetables.length === 0) {
    return convertRoutesToTimetablePages(config2);
  }
  const timetablePages = getTimetablePages(
    {},
    [],
    [["timetable_page_id", "ASC"]]
  );
  if (timetablePages.length === 0) {
    return timetables.map(
      (timetable) => convertTimetableToTimetablePage(timetable, config2)
    );
  }
  const routes = getRoutes();
  return timetablePages.map((timetablePage) => {
    timetablePage.timetables = sortBy(
      timetables.filter(
        (timetable) => timetable.timetable_page_id === timetablePage.timetable_page_id
      ),
      "timetable_sequence"
    );
    for (const timetable of timetablePage.timetables) {
      timetable.routes = routes.filter(
        (route) => timetable.route_ids.includes(route.route_id)
      );
    }
    return timetablePage;
  });
}
var getTimetablePageById = (timetablePageId, config2) => {
  const timetablePages = getTimetablePages({
    timetable_page_id: timetablePageId
  });
  const timetables = mergeTimetablesWithSameId(getTimetables());
  if (timetablePages.length > 1) {
    throw new Error(
      `Multiple timetable_pages found for timetable_page_id=${timetablePageId}`
    );
  }
  if (timetablePages.length === 1) {
    const timetablePage = timetablePages[0];
    timetablePage.timetables = sortBy(
      timetables.filter(
        (timetable) => timetable.timetable_page_id === timetablePageId
      ),
      "timetable_sequence"
    );
    for (const timetable of timetablePage.timetables) {
      timetable.routes = getRoutes({
        route_id: timetable.route_ids
      });
    }
    return timetablePage;
  }
  if (timetables.length > 0) {
    const timetablePageTimetables = timetables.filter(
      (timetable) => timetable.timetable_id === timetablePageId
    );
    if (timetablePageTimetables.length === 0) {
      throw new Error(
        `No timetable found for timetable_page_id=${timetablePageId}`
      );
    }
    return convertTimetableToTimetablePage(timetablePageTimetables[0], config2);
  }
  let calendarCode;
  let calendars;
  let calendarDates;
  let serviceId;
  let directionId = "";
  const parts = timetablePageId.split("|");
  if (parts.length > 2) {
    directionId = Number.parseInt(parts.pop(), 10);
    calendarCode = parts.pop();
  } else if (parts.length > 1) {
    directionId = null;
    calendarCode = parts.pop();
  }
  const routeId = parts.join("|");
  const routes = getRoutes({
    route_id: routeId
  });
  const trips = getTrips(
    {
      route_id: routeId,
      direction_id: directionId
    },
    ["trip_headsign", "direction_id"]
  );
  const directions = uniqBy(trips, (trip) => trip.direction_id);
  if (directions.length === 0) {
    throw new Error(
      `No trips found for timetable_page_id=${timetablePageId} route_id=${routeId} direction_id=${directionId}`
    );
  }
  if (/^[01]*$/.test(calendarCode)) {
    calendars = getCalendars({
      ...calendarCodeToCalendar(calendarCode)
    });
  } else {
    serviceId = calendarCode;
    calendarDates = getCalendarDates({
      exception_type: 1,
      service_id: serviceId
    });
  }
  return convertRouteToTimetablePage(
    routes[0],
    directions[0],
    calendars,
    calendarDates,
    config2
  );
};
function setDefaultConfig(initialConfig) {
  const defaults = {
    allowEmptyTimetables: false,
    beautify: false,
    coordinatePrecision: 5,
    dateFormat: "MMM D, YYYY",
    daysShortStrings: ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"],
    daysStrings: [
      "Monday",
      "Tuesday",
      "Wednesday",
      "Thursday",
      "Friday",
      "Saturday",
      "Sunday"
    ],
    defaultOrientation: "vertical",
    interpolatedStopSymbol: "\u2022",
    interpolatedStopText: "Estimated time of arrival",
    gtfsToHtmlVersion: version,
    linkStopUrls: false,
    menuType: "jump",
    noDropoffSymbol: "\u2021",
    noDropoffText: "No drop off available",
    noHead: false,
    noPickupSymbol: "**",
    noPickupText: "No pickup available",
    noServiceSymbol: "-",
    noServiceText: "No service at this stop",
    outputFormat: "html",
    requestDropoffSymbol: "\u2020",
    requestDropoffText: "Must request drop off",
    requestPickupSymbol: "***",
    requestPickupText: "Request stop - call for pickup",
    serviceNotProvidedOnText: "Service not provided on",
    serviceProvidedOnText: "Service provided on",
    showArrivalOnDifference: 0.2,
    showCalendarExceptions: true,
    showMap: false,
    showOnlyTimepoint: false,
    showRouteTitle: true,
    showStopCity: false,
    showStopDescription: false,
    showStoptimesForRequestStops: true,
    skipImport: false,
    sortingAlgorithm: "common",
    timeFormat: "h:mma",
    useParentStation: true,
    verbose: true,
    zipOutput: false
  };
  const config2 = Object.assign(defaults, initialConfig);
  if (config2.outputFormat === "pdf") {
    config2.noHead = false;
  }
  return config2;
}
function getFormattedTimetablePage(timetablePageId, config2) {
  const timetablePage = getTimetablePageById(
    timetablePageId,
    config2
  );
  const timetableRoutes = getRoutes(
    {
      route_id: timetablePage.route_ids
    },
    ["agency_id"]
  );
  const consolidatedTimetables = formatTimetables(
    timetablePage.timetables,
    config2
  );
  for (const timetable of consolidatedTimetables) {
    if (isNullOrEmpty(timetable.direction_name)) {
      timetable.direction_name = getDirectionHeadsignFromTimetable(timetable);
    }
    if (!timetable.routes) {
      timetable.routes = getRoutes({
        route_id: timetable.route_ids
      });
    }
  }
  const formattedTimetablePage = {
    ...timetablePage,
    consolidatedTimetables,
    dayList: formatDays(getDaysFromCalendars(consolidatedTimetables), config2),
    dayLists: uniq(
      consolidatedTimetables.map((timetable) => timetable.dayList)
    ),
    route_ids: uniq(flatMap2(consolidatedTimetables, "route_ids")),
    agency_ids: uniq(compact(timetableRoutes.map((route) => route.agency_id))),
    filename: timetablePage.filename ?? `${timetablePage.timetable_page_id}.html`
  };
  return formattedTimetablePage;
}
function generateTimetableHTML(timetablePage, config2) {
  const templateVars = {
    timetablePage,
    config: config2
  };
  return renderTemplate("timetablepage", templateVars, config2);
}
function generateOverviewHTML(timetablePages, config2) {
  const agencies = getAgencies();
  if (agencies.length === 0) {
    throw new Error("No agencies found");
  }
  let geojson;
  if (config2.showMap) {
    geojson = getAgencyGeoJSON(config2);
  }
  const sortedTimetablePages = sortBy(timetablePages, [
    (timetablePage) => {
      if (timetablePage.consolidatedTimetables.length > 0 && timetablePage.consolidatedTimetables[0].routes.length > 0) {
        return Number.parseInt(
          timetablePage.consolidatedTimetables[0].routes[0].route_short_name?.replace(
            /^\D+/g,
            ""
          ),
          10
        ) || 0;
      }
    },
    (timetablePage) => {
      if (timetablePage.consolidatedTimetables.length > 0 && timetablePage.consolidatedTimetables[0].routes.length > 0) {
        return timetablePage.consolidatedTimetables[0].routes[0].route_short_name;
      }
    }
  ]);
  const templateVars = {
    agency: {
      ...first(agencies),
      geojson
    },
    agencies,
    geojson,
    config: config2,
    timetablePages: sortedTimetablePages
  };
  return renderTemplate("overview", templateVars, config2);
}

// src/lib/formatters.ts
function replaceAll(string, mapObject) {
  const re = new RegExp(Object.keys(mapObject).join("|"), "gi");
  return string.replace(re, (matched) => mapObject[matched]);
}
function isNullOrEmpty(value) {
  return value === null || value === "";
}
function formatDate(date, dateFormat) {
  if (date.holiday_name) {
    return date.holiday_name;
  }
  return moment3(date.date, "YYYYMMDD").format(dateFormat);
}
function timeToSeconds(time) {
  return moment3.duration(time).asSeconds();
}
function formatStopTime(stoptime, timetable, config2) {
  stoptime.classes = [];
  if (stoptime.type === "arrival" && stoptime.arrival_time) {
    const arrivalTime = fromGTFSTime(stoptime.arrival_time);
    stoptime.formatted_time = arrivalTime.format(config2.timeFormat);
    stoptime.classes.push(arrivalTime.format("a"));
  } else if (stoptime.type === "departure" && stoptime.departure_time) {
    const departureTime = fromGTFSTime(stoptime.departure_time);
    stoptime.formatted_time = departureTime.format(config2.timeFormat);
    stoptime.classes.push(departureTime.format("a"));
  }
  if (stoptime.pickup_type === 1) {
    stoptime.noPickup = true;
    stoptime.classes.push("no-pickup");
    if (timetable.noPickupSymbol !== null) {
      timetable.noPickupSymbolUsed = true;
    }
  } else if (stoptime.pickup_type === 2 || stoptime.pickup_type === 3) {
    stoptime.requestPickup = true;
    stoptime.classes.push("request-pickup");
    if (timetable.requestPickupSymbol !== null) {
      timetable.requestPickupSymbolUsed = true;
    }
  }
  if (stoptime.drop_off_type === 1) {
    stoptime.noDropoff = true;
    stoptime.classes.push("no-drop-off");
    if (timetable.noDropoffSymbol !== null) {
      timetable.noDropoffSymbolUsed = true;
    }
  } else if (stoptime.drop_off_type === 2 || stoptime.drop_off_type === 3) {
    stoptime.requestDropoff = true;
    stoptime.classes.push("request-drop-off");
    if (timetable.requestDropoffSymbol !== null) {
      timetable.requestDropoffSymbolUsed = true;
    }
  }
  if (stoptime.timepoint === 0 || stoptime.departure_time === "") {
    stoptime.interpolated = true;
    stoptime.classes.push("interpolated");
    if (timetable.interpolatedStopSymbol !== null) {
      timetable.interpolatedStopSymbolUsed = true;
    }
  }
  if (stoptime.timepoint === null && stoptime.departure_time === null) {
    stoptime.skipped = true;
    stoptime.classes.push("skipped");
    if (timetable.noServiceSymbol !== null) {
      timetable.noServiceSymbolUsed = true;
    }
  }
  if (stoptime.timepoint === 1) {
    stoptime.classes.push("timepoint");
  }
  return stoptime;
}
function filterHourlyTimes(stops) {
  const firstStopTimes = [];
  const firstTripMinutes = minutesAfterMidnight(stops[0].trips[0].arrival_time);
  for (const trip of stops[0].trips) {
    const minutes = minutesAfterMidnight(trip.arrival_time);
    if (minutes >= firstTripMinutes + 60) {
      break;
    }
    firstStopTimes.push(fromGTFSTime(trip.arrival_time));
  }
  const firstStopTimesAndIndex = firstStopTimes.map((time, idx) => ({
    idx,
    time
  }));
  const sortedFirstStopTimesAndIndex = sortBy2(
    firstStopTimesAndIndex,
    (item) => Number.parseInt(item.time.format("m"), 10)
  );
  return stops.map((stop) => {
    stop.hourlyTimes = sortedFirstStopTimesAndIndex.map(
      (item) => fromGTFSTime(stop.trips[item.idx].arrival_time).format(":mm")
    );
    return stop;
  });
}
var days = [
  "monday",
  "tuesday",
  "wednesday",
  "thursday",
  "friday",
  "saturday",
  "sunday"
];
function formatDays(calendar, config2) {
  const daysShort = config2.daysShortStrings;
  let daysInARow = 0;
  let dayString = "";
  if (!calendar) {
    return "";
  }
  for (let i = 0; i <= 6; i += 1) {
    const currentDayOperating = calendar[days[i]] === 1;
    const previousDayOperating = i > 0 ? calendar[days[i - 1]] === 1 : false;
    const nextDayOperating = i < 6 ? calendar[days[i + 1]] === 1 : false;
    if (currentDayOperating) {
      if (dayString.length > 0) {
        if (!previousDayOperating) {
          dayString += ", ";
        } else if (daysInARow === 1) {
          dayString += "-";
        }
      }
      daysInARow += 1;
      if (dayString.length === 0 || !nextDayOperating || i === 6 || !previousDayOperating) {
        dayString += daysShort[i];
      }
    } else {
      daysInARow = 0;
    }
  }
  if (dayString.length === 0) {
    dayString = "No regular service days";
  }
  return dayString;
}
function formatDaysLong(dayList, config2) {
  const mapObject = zipObject(config2.daysShortStrings, config2.daysStrings);
  return replaceAll(dayList, mapObject);
}
function formatTrip(trip, timetable, calendars, config2) {
  trip.calendar = find2(calendars, {
    service_id: trip.service_id
  });
  trip.dayList = formatDays(trip.calendar, config2);
  trip.dayListLong = formatDaysLong(trip.dayList, config2);
  if (timetable.routes.length === 1) {
    trip.route_short_name = timetable.routes[0].route_short_name;
  } else {
    const route = timetable.routes.find(
      (route2) => route2.route_id === trip.route_id
    );
    trip.route_short_name = route.route_short_name;
  }
  return trip;
}
function formatFrequency(frequency, config2) {
  const startTime = fromGTFSTime(frequency.start_time);
  const endTime = fromGTFSTime(frequency.end_time);
  const headway = moment3.duration(frequency.headway_secs, "seconds");
  frequency.start_formatted_time = startTime.format(config2.timeFormat);
  frequency.end_formatted_time = endTime.format(config2.timeFormat);
  frequency.headway_min = Math.round(headway.asMinutes());
  return frequency;
}
function formatTimetableId(timetable) {
  let timetableId = `${timetable.route_ids.join("_")}|${calendarToCalendarCode(
    timetable
  )}`;
  if (!isNullOrEmpty(timetable.direction_id)) {
    timetableId += `|${timetable.direction_id}`;
  }
  return timetableId;
}
function createEmptyStoptime(stopId, tripId) {
  return {
    id: null,
    trip_id: tripId,
    arrival_time: null,
    departure_time: null,
    stop_id: stopId,
    stop_sequence: null,
    stop_headsign: null,
    pickup_type: null,
    drop_off_type: null,
    continuous_pickup: null,
    continuous_drop_off: null,
    shape_dist_traveled: null,
    timepoint: null
  };
}
function formatStops(timetable, config2) {
  for (const trip of timetable.orderedTrips) {
    let stopIndex = -1;
    for (const [idx, stoptime] of trip.stoptimes.entries()) {
      const stop = find2(timetable.stops, (st, idx2) => {
        if (st.stop_id === stoptime.stop_id && idx2 > stopIndex) {
          stopIndex = idx2;
          return true;
        }
        return false;
      });
      if (!stop) {
        continue;
      }
      if (idx === 0) {
        stoptime.drop_off_type = 0;
      }
      if (idx === trip.stoptimes.length - 1) {
        stoptime.pickup_type = 0;
      }
      if (stop.type === "arrival" && idx < trip.stoptimes.length - 1) {
        const departureStoptime = clone(stoptime);
        departureStoptime.type = "departure";
        timetable.stops[stopIndex + 1].trips.push(
          formatStopTime(departureStoptime, timetable, config2)
        );
      }
      if (!(stop.type === "arrival" && idx === 0)) {
        stoptime.type = "arrival";
        stop.trips.push(formatStopTime(stoptime, timetable, config2));
      }
    }
    for (const stop of timetable.stops) {
      const lastStopTime = last2(stop.trips);
      if (!lastStopTime || lastStopTime.trip_id !== trip.trip_id) {
        stop.trips.push(
          formatStopTime(
            createEmptyStoptime(stop.stop_id, trip.trip_id),
            timetable,
            config2
          )
        );
      }
    }
  }
  if (timetable.orientation === "hourly") {
    timetable.stops = filterHourlyTimes(timetable.stops);
  }
  for (const stop of timetable.stops) {
    stop.is_timepoint = stop.trips.some((stoptime) => isTimepoint(stoptime));
  }
  return timetable.stops;
}
function resetStoptimesToMidnight(trip) {
  const offsetSeconds = secondsAfterMidnight(
    first2(trip.stoptimes).departure_time
  );
  if (offsetSeconds > 0) {
    for (const stoptime of trip.stoptimes) {
      stoptime.departure_time = toGTFSTime(
        fromGTFSTime(stoptime.departure_time).subtract(
          offsetSeconds,
          "seconds"
        )
      );
      stoptime.arrival_time = toGTFSTime(
        fromGTFSTime(stoptime.arrival_time).subtract(offsetSeconds, "seconds")
      );
    }
  }
  return trip;
}
function updateStoptimesByOffset(trip, offsetSeconds) {
  return trip.stoptimes.map((stoptime) => {
    delete stoptime._id;
    stoptime.departure_time = updateTimeByOffset(
      stoptime.departure_time,
      offsetSeconds
    );
    stoptime.arrival_time = updateTimeByOffset(
      stoptime.arrival_time,
      offsetSeconds
    );
    stoptime.trip_id = trip.trip_id;
    return stoptime;
  });
}
function formatRouteColor(route) {
  return route.route_color ? `#${route.route_color}` : "#000000";
}
function formatRouteTextColor(route) {
  return route.route_text_color ? `#${route.route_text_color}` : "#FFFFFF";
}
function formatTimetableLabel(timetable) {
  if (!isNullOrEmpty(timetable.timetable_label)) {
    return timetable.timetable_label;
  }
  let timetableLabel = "";
  if (timetable.routes && timetable.routes.length > 0) {
    timetableLabel += "Route ";
    if (!isNullOrEmpty(timetable.routes[0].route_short_name)) {
      timetableLabel += timetable.routes[0].route_short_name;
    } else if (!isNullOrEmpty(timetable.routes[0].route_long_name)) {
      timetableLabel += timetable.routes[0].route_long_name;
    }
  }
  if (timetable.stops && timetable.stops.length > 0) {
    const firstStop = timetable.stops[0].stop_name;
    const lastStop = timetable.stops[timetable.stops.length - 1].stop_name;
    if (firstStop === lastStop) {
      if (!isNullOrEmpty(timetable.routes[0].route_long_name)) {
        timetableLabel += ` - ${timetable.routes[0].route_long_name}`;
      }
      timetableLabel += " - Loop";
    } else {
      timetableLabel += ` - ${firstStop} to ${lastStop}`;
    }
  } else if (timetable.direction_name !== null) {
    timetableLabel += ` to ${timetable.direction_name}`;
  }
  return timetableLabel;
}
function mergeTimetablesWithSameId(timetables) {
  if (timetables.length === 0) {
    return [];
  }
  const mergedTimetables = groupBy2(timetables, "timetable_id");
  return Object.values(mergedTimetables).map((timetableGroup) => {
    const mergedTimetable = omit(timetableGroup[0], "route_id");
    mergedTimetable.route_ids = timetableGroup.map(
      (timetable) => timetable.route_id
    );
    return mergedTimetable;
  });
}

// src/app/index.ts
var argv = yargs(hideBin(process.argv)).option("c", {
  alias: "configPath",
  describe: "Path to config file",
  default: "./config.json",
  type: "string"
}).parseSync();
var app = express();
var router = express.Router();
var configPath = argv.configPath || new URL("../../config.json", import.meta.url);
var selectedConfig = JSON.parse(readFileSync(configPath, "utf8"));
var config = setDefaultConfig(selectedConfig);
config.noHead = false;
config.assetPath = "/";
config.log = console.log;
config.logWarning = console.warn;
config.logError = console.error;
try {
  openDb2(config);
} catch (error) {
  if (error?.code === "SQLITE_CANTOPEN") {
    config.logError(
      `Unable to open sqlite database "${config.sqlitePath}" defined as \`sqlitePath\` config.json. Ensure the parent directory exists or remove \`sqlitePath\` from config.json.`
    );
  }
  throw error;
}
router.get("/", async (request, response, next) => {
  try {
    const timetablePages = [];
    const timetablePageIds = map(
      getTimetablePagesForAgency(config),
      "timetable_page_id"
    );
    for (const timetablePageId of timetablePageIds) {
      const timetablePage = await getFormattedTimetablePage(
        timetablePageId,
        config
      );
      if (!timetablePage.consolidatedTimetables || timetablePage.consolidatedTimetables.length === 0) {
        console.error(
          `No timetables found for timetable_page_id=${timetablePage.timetable_page_id}`
        );
      }
      timetablePage.relativePath = `/timetables/${timetablePage.timetable_page_id}`;
      for (const timetable of timetablePage.consolidatedTimetables) {
        timetable.timetable_label = formatTimetableLabel(timetable);
      }
      timetablePages.push(timetablePage);
    }
    const html = await generateOverviewHTML(timetablePages, config);
    response.send(html);
  } catch (error) {
    console.error(error);
    next(error);
  }
});
router.get("/timetables/:timetablePageId", async (request, response, next) => {
  const { timetablePageId } = request.params;
  if (!timetablePageId) {
    return next(new Error("No timetablePageId provided"));
  }
  try {
    const timetablePage = await getFormattedTimetablePage(
      timetablePageId,
      config
    );
    const html = await generateTimetableHTML(timetablePage, config);
    response.send(html);
  } catch (error) {
    next(error);
  }
});
app.set("views", path2.join(fileURLToPath2(import.meta.url), "../../../views"));
app.set("view engine", "pug");
app.use(logger("dev"));
var staticAssetPath = config.templatePath === void 0 ? path2.join(fileURLToPath2(import.meta.url), "../../../views/default") : untildify2(config.templatePath);
app.use(express.static(staticAssetPath));
app.use("/", router);
app.set("port", process.env.PORT || 3e3);
var server = app.listen(app.get("port"), () => {
  console.log(`Express server listening on port ${app.get("port")}`);
});
//# sourceMappingURL=index.js.map